<?php

namespace Ulmus\Ldap;

use Ulmus\{Attribute\Property\Field,
    ConnectionAdapter,
    Repository,
    Ulmus,
    EventTrait,
    Query,
    Common\EntityResolver,
    Common\EntityField};

use Notes\Attribute\Ignore;
use Ulmus\Ldap\Annotation\Classes\{ ObjectClass, ObjectType, };

trait EntityTrait {
    use \Ulmus\EntityTrait;

    #[Field\Id(readonly: true)]
    public string $dn;

    #[Field]
    public string $cn;

    #[Field]
    public array $objectClass;

    #[Ignore]
    public static function resolveEntity() : EntityResolver
    {
        return Ulmus::resolveEntity(static::class);
    }

    #[Ignore]
    public static function repository(string $alias = Repository::DEFAULT_ALIAS, ConnectionAdapter $adapter = null) : Repository
    {
        return Ulmus::repository(static::class, $alias, $adapter);
    }

    #[Ignore]
    public static function field($name, null|string|false $alias = Repository::DEFAULT_ALIAS) : EntityField
    {
        return new EntityField(static::class, $name, false, Ulmus::resolveEntity(static::class));
    }

    #[Ignore]
    public static function fields(array $fields, null|string|false $alias = Repository::DEFAULT_ALIAS, string $separator = ', ') : string
    {
        return implode(', ', array_map(function($item) {
            return static::field($item, false);
        }, $fields));
    }
}
